package cn.newfeifan.mall.module.distri.service.ptdailybill;

import org.springframework.stereotype.Service;
import javax.annotation.Resource;
import org.springframework.validation.annotation.Validated;
import cn.newfeifan.mall.module.distri.controller.admin.ptdailybill.vo.*;
import cn.newfeifan.mall.module.distri.dal.dataobject.ptdailybill.PtDailyBillDO;
import cn.newfeifan.mall.framework.common.pojo.PageResult;
import cn.newfeifan.mall.framework.common.util.object.BeanUtils;

import cn.newfeifan.mall.module.distri.dal.mysql.ptdailybill.PtDailyBillMapper;

import static cn.newfeifan.mall.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.newfeifan.mall.module.distri.enums.ErrorCodeConstants.*;

/**
 * 平台每日账单统计 Service 实现类
 *
 * @author 非繁人
 */
@Service
@Validated
public class PtDailyBillServiceImpl implements PtDailyBillService {

    @Resource
    private PtDailyBillMapper ptDailyBillMapper;

    @Override
    public Long createPtDailyBill(PtDailyBillSaveReqVO createReqVO) {
        // 插入
        PtDailyBillDO ptDailyBill = BeanUtils.toBean(createReqVO, PtDailyBillDO.class);
        ptDailyBillMapper.insert(ptDailyBill);
        // 返回
        return ptDailyBill.getId();
    }

    @Override
    public void updatePtDailyBill(PtDailyBillSaveReqVO updateReqVO) {
        // 校验存在
        validatePtDailyBillExists(updateReqVO.getId());
        // 更新
        PtDailyBillDO updateObj = BeanUtils.toBean(updateReqVO, PtDailyBillDO.class);
        ptDailyBillMapper.updateById(updateObj);
    }

    @Override
    public void deletePtDailyBill(Long id) {
        // 校验存在
        validatePtDailyBillExists(id);
        // 删除
        ptDailyBillMapper.deleteById(id);
    }

    private void validatePtDailyBillExists(Long id) {
        if (ptDailyBillMapper.selectById(id) == null) {
            throw exception(PT_DAILY_BILL_NOT_EXISTS);
        }
    }

    @Override
    public PtDailyBillDO getPtDailyBill(Long id) {
        return ptDailyBillMapper.selectById(id);
    }

    @Override
    public PageResult<PtDailyBillDO> getPtDailyBillPage(PtDailyBillPageReqVO pageReqVO) {
        return ptDailyBillMapper.selectPage(pageReqVO);
    }

}